@@ -1,5 +1,5 @@
 /*
-  $Id: icewm.cpp,v 1.37 2004/01/09 16:38:08 giessl Exp $
+  $Id: icewm.cpp,v 1.42.2.1 2004/11/25 16:09:56 lunakl Exp $
 
   Gallium-IceWM themeable KWin client
 
@@ -101,6 +101,8 @@ QString* titleButtonsRight;
 
 QColor* colorActiveBorder;
 QColor* colorInActiveBorder;
+QColor* colorActiveButton;
+QColor* colorInActiveButton;
 QColor* colorActiveTitleBarText;
 QColor* colorInActiveTitleBarText;
 QColor* colorActiveTitleBar;
@@ -127,7 +129,7 @@ bool showMenuButtonIcon   	= false;	// Draw a mini icon over the menu pixmap.
 bool customButtonPositions 	= false;	// Let the theme dictate the btn pos.
 bool titleBarCentered 	  	= true;
 
-
+enum styles {OTHER, WARP3, WARP4, MOTIF, WIN95, NICE} themeLook;
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 // General utility functions
@@ -159,6 +161,8 @@ ThemeHandler::ThemeHandler()
 
 	colorActiveBorder 			= new QColor();
 	colorInActiveBorder 		= new QColor();
+	colorActiveButton 	        = new QColor();
+	colorInActiveButton 		= new QColor();
 	colorActiveTitleBarText 	= new QColor();
 	colorInActiveTitleBarText	= new QColor();
 	colorActiveTitleBar 		= new QColor();
@@ -187,6 +191,8 @@ ThemeHandler::~ThemeHandler()
 	delete colorActiveTitleBar;
 	delete colorInActiveTitleBar;
 	delete colorActiveBorder;
+	delete colorActiveButton;
+	delete colorInActiveButton;
 
 	delete titleButtonsRight;
 	delete titleButtonsLeft;
@@ -287,7 +293,7 @@ void ThemeHandler::initTheme()
 		// (some themes look bad with it on by default)
 		*titleButtonsLeft = config.readEntry("TitleButtonsLeft", "s");
 		*titleButtonsLeft = titleButtonsLeft->replace( QRegExp(QString("\"")), "");
-		*titleButtonsRight = config.readEntry("TitleButtonsRight", "xmi");
+		*titleButtonsRight = config.readEntry("TitleButtonsRight", "xmir");
 		*titleButtonsRight = titleButtonsRight->replace( QRegExp(QString("\"")), "");
 
 		// I have no idea why the right side buttons in icewm are reversed
@@ -297,10 +303,23 @@ void ThemeHandler::initTheme()
 	// Read the default border and text colours from the config file
 	// And use IceWM defaults if not found
 	QString s;
+
+	s = config.readEntry("Look", "other");
+	if (s=="motif") themeLook = MOTIF;
+	else if (s=="warp3") themeLook = WARP3;
+	else if (s=="warp4") themeLook = WARP4;
+	else if (s=="win95") themeLook = WIN95;
+	else if (s=="nice") themeLook = NICE;
+	else themeLook = OTHER;
+
 	s = config.readEntry("ColorActiveBorder", "#C0C0C0");
 	*colorActiveBorder = decodeColor( s );
 	s = config.readEntry("ColorNormalBorder", "#C0C0C0");
 	*colorInActiveBorder = decodeColor( s );
+	s = config.readEntry("ColorActiveButton", "#C0C0C0");
+	*colorActiveButton = decodeColor( s );
+	s = config.readEntry("ColorNormalButton", "#C0C0C0");
+	*colorInActiveButton = decodeColor( s );
 
 	// Use these as a last resort
 	s = config.readEntry("ColorActiveTitleBar", "#0000A0");
@@ -350,15 +369,15 @@ void ThemeHandler::initTheme()
 	setPixmap( titleR, "title", "R.xpm" );
 	setPixmap( titleQ, "title", "Q.xpm" );
 
-	setPixmap( closePix,     "close",     ".xpm" );
-	setPixmap( depthPix,     "depth",     ".xpm" );
-	setPixmap( maximizePix,  "maximize",  ".xpm" );
-	setPixmap( minimizePix,  "minimize",  ".xpm" );
-	setPixmap( restorePix,   "restore",   ".xpm" );
-	setPixmap( hidePix,      "hide",      ".xpm" );
-	setPixmap( rollupPix,    "rollup",    ".xpm" );
-	setPixmap( rolldownPix,  "rolldown",  ".xpm" );
-	setPixmap( menuButtonPix,"menuButton",".xpm" );
+	setPixmapButton( closePix,     "close",     ".xpm" );
+	setPixmapButton( depthPix,     "depth",     ".xpm" );
+	setPixmapButton( maximizePix,  "maximize",  ".xpm" );
+	setPixmapButton( minimizePix,  "minimize",  ".xpm" );
+	setPixmapButton( restorePix,   "restore",   ".xpm" );
+	setPixmapButton( hidePix,      "hide",      ".xpm" );
+	setPixmapButton( rollupPix,    "rollup",    ".xpm" );
+	setPixmapButton( rolldownPix,  "rolldown",  ".xpm" );
+	setPixmapButton( menuButtonPix,"menuButton",".xpm" );
 
 	// Top
 	setPixmap( frameTL, "frame", "TL.xpm" );
@@ -547,6 +566,114 @@ QPixmap* ThemeHandler::stretchPixmap( QPixmap* src, bool stretchHoriz, int stret
 	return p;
 }
 
+static void draw3DRect(QPainter &pnt, QColor &col, int x, int y, int w, int h, bool up) {
+	QColor light = col.light(135);
+	QColor dark = col.dark(140);
+	pnt.setPen(up ? light : dark);
+	pnt.drawLine(x, y, x+w, y);
+	pnt.drawLine(x, y, x, y+h);
+	pnt.setPen(up ? dark : light);
+	pnt.drawLine(x, y+h, x+w, y+h);
+	pnt.drawLine(x+w, y, x+w, y+h);
+	pnt.setPen(col);
+	pnt.drawPoint(x+w, y);
+	pnt.drawPoint(x, y+h);
+}
+
+void ThemeHandler::setPixmapButton( QPixmap* p[], QString s1, QString s2)
+{
+	if ( p[Active] )
+		qWarning("kwin-icewm: setPixmap - should be null (1)\n");
+	if ( p[InActive] )
+		qWarning("kwin-icewm: setPixmap - should be null (2)\n");
+
+	QString str = locate("appdata", QString("icewm-themes/")
+				+ themeName + s1 + "A" + s2);
+	if (str == QString::null)
+		str = locate("appdata", QString("icewm-themes/")
+				+ themeName + s1 + s2);
+
+        QPixmap *qp = new QPixmap(str);
+	QColor cActive = themeLook == WIN95 ? *colorActiveTitleBar : *colorActiveButton;
+	QColor cInActive = themeLook == WIN95 ? *colorInActiveTitleBar : *colorInActiveButton;
+
+	if (!qp->isNull() && themeLook > 0) {
+		int w = qp->width();
+		if (themeLook > 0 && titleBarHeight > w) w = titleBarHeight;
+		p[Active] = new QPixmap(w, 2*titleBarHeight );
+		p[Active] -> fill(cActive);
+
+		QPainter pnt( p[Active] );
+
+		int offX = (w - qp->width())/2;
+		int offY = (titleBarHeight - qp->height())/2;
+		if (offY < 0) offY = 0;
+
+		if (themeLook == WIN95) {
+			draw3DRect(pnt, *colorActiveButton, offX-1, offY-1,
+				   qp->width()+1, qp->height()+1, true);
+			draw3DRect(pnt, *colorActiveButton, offX-1, offY-1 + titleBarHeight,
+				   qp->width()+1, qp->height()+1, false);
+		} else if (themeLook != WARP4) {
+			draw3DRect(pnt, *colorActiveButton, 0, 0,
+				   w-1, titleBarHeight-1, true);
+			draw3DRect(pnt, *colorActiveButton, 0, titleBarHeight,
+				   w-1, 2*titleBarHeight-1, false);
+		}
+
+		pnt.drawPixmap(offX, offY, *qp);
+		if (qp->height() <= titleBarHeight) {
+			pnt.drawPixmap(offX, titleBarHeight+offY, *qp);
+		}
+		pnt.end();
+		delete qp;
+	} else {
+		p[Active] = qp;
+	}
+
+	str = locate("appdata", QString("icewm-themes/")
+		     + themeName + s1 + "I" + s2);
+	if (str == QString::null)
+		str = locate("appdata", QString("icewm-themes/")
+			     + themeName + s1 + s2);
+
+        qp = new QPixmap(str);
+	if (!qp->isNull() && themeLook > 0) {
+		int w = qp->width();
+		if (titleBarHeight > w) w = titleBarHeight;
+		p[InActive] = new QPixmap(w, 2*titleBarHeight );
+		p[InActive] -> fill(cInActive);
+
+		QPainter pnt( p[InActive] );
+
+		int offX = (w - qp->width())/2;
+		int offY = (titleBarHeight - qp->height())/2;
+		if (offY < 0) offY = 0;
+
+		if (themeLook == WIN95) {
+			draw3DRect(pnt, *colorInActiveButton, offX-1, offY-1,
+				   qp->width()+1, qp->height()+1, true);
+			draw3DRect(pnt, *colorInActiveButton, offX-1, offY-1 + titleBarHeight,
+				   qp->width()+1, qp->height()+1, false);
+ 		} else if (themeLook != WARP4) {
+			draw3DRect(pnt, *colorInActiveButton, 0, 0,
+				   w-1, titleBarHeight-1, true);
+			draw3DRect(pnt, *colorActiveButton, 0, titleBarHeight,
+				   w-1, 2*titleBarHeight-1, false);
+		}
+		pnt.drawPixmap(offX, offY, *qp);
+		if (qp->height() <= titleBarHeight) {
+			pnt.drawPixmap(offX, titleBarHeight+offY, *qp);
+		}
+		pnt.end();
+		delete qp;
+	} else {
+		p[InActive] = qp;
+	}
+}
+
+
+
 
 // Loads the specified Active/InActive files into the specific pixmaps, and
 // can perform horizontal / vertical stretching if required for speed.
@@ -826,7 +953,7 @@ void IceWMClient::addClientButtons( const QString& s )
 				case 's':
 					// Create the menu icons, and render with the current mini-icon
 					// if explicitly requested by the theme.
-					if ( validPixmaps(menuButtonPix) && !button[BtnSysMenu])
+					if ( (validPixmaps(menuButtonPix) || showMenuButtonIcon) && !button[BtnSysMenu])
 					{
 						if (showMenuButtonIcon) {
 							renderMenuIcons();
@@ -885,26 +1012,25 @@ void IceWMClient::addClientButtons( const QString& s )
 						hb->addWidget( button[BtnHide] );
 					break; */
 
-				/* Re-enable this when kwin has void shadeChange(bool s)
-				   in clients.cpp
 				case 'r':
 					// NOTE: kwin doesn't have toggleShade() in clients.h !
 				    if ( validPixmaps(rollupPix) && !button[BtnRollup] )
 				    {
 						button[BtnRollup] = new IceWMButton(this, "shade",
-							isShade() ? &rolldownPix : &rollupPix);
-						connect( button[BtnRollup], SIGNAL(clicked()),
-								 this, SLOT(toggleShade()) );
+							isSetShade() ? &rolldownPix : &rollupPix,
+										    false, i18n("Rollup"));
 						hb->addWidget( button[BtnRollup] );
+						connect( button[BtnRollup], SIGNAL(clicked()),
+								 this, SLOT(toggleShade()));
 					}
-					break; 	*/
+					break;
 
 				case 'd':
 					// Make depth == on all desktops
 					if ( validPixmaps(depthPix) && !button[BtnDepth] )
 					{
 						button[BtnDepth] = new IceWMButton(this, "on_all_desktops",
-								&depthPix, true, isOnAllDesktops()?i18n("Not On All Desktops"):i18n("On All Desktops"));
+								&depthPix, true, isOnAllDesktops()?i18n("Not on all desktops"):i18n("On all desktops"));
 						button[BtnDepth]->turnOn( isOnAllDesktops() );
 						hb->addWidget( button[BtnDepth] );
 						connect( button[BtnDepth], SIGNAL(clicked()),
@@ -938,43 +1064,53 @@ QSpacerItem* IceWMClient::addPixmapSpacer( QPixmap* p[], QSizePolicy::SizeType s
 void IceWMClient::renderMenuIcons()
 {
 	QPixmap miniIcon( icon().pixmap( QIconSet::Small, QIconSet::Normal) );
-	if (validPixmaps(menuButtonPix) && (!miniIcon.isNull()))
-		for(int i = 0; i < 2; i++)
-		{
-			if ( menuButtonWithIconPix[i] )
-				 delete menuButtonWithIconPix[i];
 
-			// Try to be more friendly to dodgy themes - icewm assumes a square menu button
-			// but some pixmap themes don't provide a square menu button.
-	        menuButtonWithIconPix[i] = new QPixmap( titleBarHeight, 2*titleBarHeight );
+	if (!miniIcon.isNull())
+	         for(int i = 0; i < 2; i++) {
+                       if ( menuButtonWithIconPix[i] )
+                              delete menuButtonWithIconPix[i];
 
-			QPainter pnt( menuButtonWithIconPix[i] );
-			pnt.drawPixmap(0, 0, *menuButtonPix[i]);
+                       // Try to be more friendly to dodgy themes - icewm assumes a square menu button
+                       // but some pixmap themes don't provide a square menu button.
+                       int w = titleBarHeight;
+                       if (validPixmaps(menuButtonPix) && menuButtonPix[i]->width() > w)
+                               w = menuButtonPix[i]->width();
+                       menuButtonWithIconPix[i] = new QPixmap(w, 2*titleBarHeight );
+                       if (themeLook != WIN95)
+                               menuButtonWithIconPix[i] -> fill((i==0) ? *colorInActiveButton : *colorActiveButton);
+                       else
+                               menuButtonWithIconPix[i] -> fill((i==0) ? *colorInActiveTitleBar : *colorActiveTitleBar);
+                       QPainter pnt( menuButtonWithIconPix[i] );
+
+                       if (themeLook > 0 && themeLook != WIN95 && themeLook != WARP4) {
+                               draw3DRect(pnt, *colorActiveButton, 0, 0,
+                                          w-1, titleBarHeight-1, true);
+                               draw3DRect(pnt, *colorActiveButton, 0, titleBarHeight,
+                                          w-1, 2*titleBarHeight-1, false);
+                       }
+                       if (validPixmaps(menuButtonPix)) {
+                               pnt.drawPixmap(0, 0, *menuButtonPix[i]);
+                       }
+                       int offset = (titleBarHeight - miniIcon.width())/2;
+                       if (offset<0) offset = 0;
+                       // Paint the mini icon over the menu pixmap in the centre
+                       pnt.drawPixmap( offset, offset, miniIcon );
+                       pnt.drawPixmap( offset, titleBarHeight+offset, miniIcon );
+                       pnt.end();
+                }
 
-			int offset = (titleBarHeight - miniIcon.width())/2;
-			// Paint the mini icon over the menu pixmap in the centre
-			pnt.drawPixmap( offset, offset, miniIcon );
-			pnt.drawPixmap( offset, titleBarHeight+offset, miniIcon );
-			pnt.end();
-		}
 }
 
 
 void IceWMClient::slotMaximize()
 {
-	switch (button[BtnMaximize]->last_button)
-	{
-		case MidButton:
-			maximize(maximizeMode() ^ MaximizeVertical );
-			break;
-		case RightButton:
-			maximize(maximizeMode() ^ MaximizeHorizontal );
-			break;
-		default:
-			maximize(maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull );
-	}
+	maximize(button[BtnMaximize]->last_button);
 }
 
+void IceWMClient::toggleShade()
+{
+        setShade(!isSetShade());
+}
 
 int IceWMClient::titleTextWidth( const QString& s )
 {
@@ -1128,21 +1264,29 @@ void IceWMClient::paintEvent( QPaintEvent* )
 	{
 		// Draw a stock IceWM frame instead of a pixmap frame
 		c1 = isActive() ? *colorActiveBorder : *colorInActiveBorder;
-		p.setPen( c1.light(135) );
-		p.drawLine(0, 0, w-2, 0);
-		p.drawLine(0, 0, 0, h-2);
 
-		p.setPen(c1);
-		p.drawLine(1, 1, w-3, 1);
-		p.drawLine(1, 1, 1, h-3);
+		if (themeLook == WARP3 || themeLook == MOTIF) {
+			draw3DRect(p, c1, x, y, w-1, h-1, true);
+			p.setPen(c1);
+			p.drawRect(x+1, y+1, w-2, h-2);
+		} else {
+			p.setPen( c1.light(135) );
+			p.drawLine(0, 0, w-2, 0);
+			p.drawLine(0, 0, 0, h-2);
 
-		p.setPen( c1.dark(140) );
-		p.drawLine(1, h-2, w-2, h-2);
-		p.drawLine(w-2, 1, w-2, h-2);
+			p.setPen(c1);
+			p.drawLine(1, 1, w-3, 1);
+			p.drawLine(1, 1, 1, h-3);
+
+			p.setPen( c1.dark(140) );
+			p.drawLine(1, h-2, w-2, h-2);
+			p.drawLine(w-2, 1, w-2, h-2);
+
+			p.setPen( Qt::black );
+			p.drawLine(w-1, 0, w-1, h-1);
+			p.drawLine(0, h-1, w-1, h-1);
+		}
 
-		p.setPen( Qt::black );
-		p.drawLine(w-1, 0, w-1, h-1);
-		p.drawLine(0, h-1, w-1, h-1);
 
 		// Fill frame border if required
 		if (borderSizeX > 2)
@@ -1151,6 +1295,7 @@ void IceWMClient::paintEvent( QPaintEvent* )
 			p.fillRect( x+2, y+2, borderSizeX-2, h-4, c1);
 			p.fillRect( w-borderSizeX, y+2, borderSizeX-2, h-4, c1);
 		}
+
 		if (borderSizeY > 2)
 		{
 			// Fill horizontal frame parts
@@ -1158,6 +1303,48 @@ void IceWMClient::paintEvent( QPaintEvent* )
 			p.fillRect( x+borderSizeX, h-borderSizeY, fillWidth, borderSizeY-2, c1);
 		}
 
+		if (themeLook == WARP3 || themeLook == MOTIF) {
+			draw3DRect(p, c1, x+borderSizeX-1, y+borderSizeY-1,
+				    w+1-2*borderSizeX, h+1-2*borderSizeY, false);
+		}
+		if (themeLook == MOTIF && !isShade()) {
+			int xext = titleBarHeight + borderSizeX - 1;
+			int yext = titleBarHeight + borderSizeY - 1;
+
+			int xext2 = w-xext-2;
+			int yext2 = h-yext-2;
+
+			int bX = w - borderSizeX-1;
+			int bY = h - borderSizeY-1;
+
+			p.setPen( c1.dark(140) );
+			p.drawLine(xext, 0, xext, borderSizeY);
+			p.drawLine(xext2, 0, xext2, borderSizeY);
+			p.drawLine(xext, bY, xext, h-1);
+			p.drawLine(xext2, bY, xext2, h-1);
+
+			p.drawLine(0, yext, borderSizeX, yext);
+			p.drawLine(0, yext2, borderSizeX, yext2);
+			p.drawLine(bX, yext, w-1, yext);
+			p.drawLine(bX, yext2, w-1, yext2);
+
+			p.setPen( c1.light(135) );
+
+			++xext; ++yext; ++xext2; ++yext2;
+
+			p.drawLine(xext, 0, xext, borderSizeY);
+			p.drawLine(xext2, 0, xext2, borderSizeY);
+			p.drawLine(xext, bY, xext, h-1);
+			p.drawLine(xext2, bY, xext2, h-1);
+
+			p.drawLine(0, yext, borderSizeX, yext);
+			p.drawLine(0, yext2, borderSizeX, yext2);
+			p.drawLine(bX, yext, w-1, yext);
+			p.drawLine(bX, yext2, w-1, yext2);
+
+
+		}
+
 		// Ensure uncovered areas during shading are painted with something
 		p.setPen( *colorInActiveBorder );
 		if (titleBarOnTop)
@@ -1287,7 +1474,7 @@ void IceWMClient::desktopChange()
 	{
 		button[BtnDepth]->turnOn( isOnAllDesktops() );
 		button[BtnDepth]->repaint(false);
-		button[BtnDepth]->setTipText(isOnAllDesktops() ? i18n("Not On All Desktops") : i18n("On All Desktops"));
+		button[BtnDepth]->setTipText(isOnAllDesktops() ? i18n("Not on all desktops") : i18n("On all desktops"));
 	}
 }
 
@@ -1316,14 +1503,16 @@ void IceWMClient::maximizeChange()
 }
 
 
-/* Re-enable this when kwin has void shadeChange(bool s) in clients.cpp
-void IceWMClient::shadeChange(bool s)
+void IceWMClient::shadeChange()
 {
 	// Change the button pixmap to rolldown if required
 	if (button[BtnRollup] && validPixmaps(rolldownPix))
-		button[BtnRollup]->usePixmap( s ? rolldownPix : rollupPix );
+	{
+		button[BtnRollup]->usePixmap( isSetShade() ? &rolldownPix : &rollupPix );
+		button[BtnRollup]->setTipText( isSetShade() ? i18n("Rolldown") : i18n("Rollup"));
+	}
+
 }
-*/
 
 
 void IceWMClient::activeChange()
@@ -1344,9 +1533,9 @@ void IceWMClient::calcHiddenButtons()
 	const int minwidth = 220; // Minimum width where all buttons are shown
 	const int btn_width = 20; // Average width
 
-	// Show/Hide buttons in this order - OnAllDesktops, Maximize, Menu, Minimize, Close.
+	// Show/Hide buttons in this order - OnAllDesktops, Maximize, Menu, Rollup, Minimize, Close.
 	IceWMButton* btnArray[] = { button[BtnDepth], button[BtnMaximize], button[BtnSysMenu],
-								button[BtnMinimize], button[BtnClose] };
+								button[BtnRollup], button[BtnMinimize], button[BtnClose] };
 
 	int current_width = width();
 	int count = 0;
@@ -1360,7 +1549,7 @@ void IceWMClient::calcHiddenButtons()
 	}
 
 	// Bound the number of buttons to hide
-	if (count > 5) count = 5;
+	if (count > 6) count = 6;
 
 	// Hide the required buttons...
 	for(i = 0; i < count; i++)
@@ -1370,7 +1559,7 @@ void IceWMClient::calcHiddenButtons()
 	}
 
 	// Show the rest of the buttons...
-	for(i = count; i < 5; i++)
+	for(i = count; i < 6; i++)
 	{
 		if (btnArray[i] && (!btnArray[i]->isVisible()) )
 			btnArray[i]->show();
