@@ -36,6 +36,98 @@ static const unsigned char maximize_bits[] = {
    0x30, 0x00, 0x78, 0x00, 0xfc, 0x00, 0xfe, 0x01, 0x00, 0x00, 0xfe, 0x01,
    0x02, 0x01, 0x84, 0x00, 0x48, 0x00, 0x30, 0x00 };
 
+static const unsigned char shade_bits[] = {
+    0xff,0x03,
+    0xff,0x03,
+    0x03,0x03,
+    0xff,0x03,
+    0xff,0x03,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00 
+};
+    
+static const unsigned char unshade_bits[] = {
+    0xff,0x03,
+    0xff,0x03,
+    0x03,0x03,
+    0x03,0x03,
+    0x03,0x03,
+    0x03,0x03,
+    0x03,0x03,
+    0x03,0x03,
+    0xff,0x03,
+    0xff,0x03 
+};
+
+static const unsigned char keep_above_bits[] = {
+    0x30,0x00,
+    0x78,0x00,
+    0xfc,0x00,
+    0x00,0x00,
+    0xff,0x03,
+    0xff,0x03,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00 
+};
+
+static const unsigned char from_above_bits[] = {
+    0xff,0x03,
+    0xff,0x03,
+    0x00,0x00,
+    0xfc,0x00,
+    0x78,0x00,
+    0x30,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00 
+};
+    
+static const unsigned char keep_below_bits[] = {
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0xff,0x03,
+    0xff,0x03,
+    0x00,0x00,
+    0xfc,0x00,
+    0x78,0x00,
+    0x30,0x00
+};
+
+static const unsigned char from_below_bits[] = {
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x00,0x00,
+    0x30,0x00,
+    0x78,0x00,
+    0xfc,0x00,
+    0x00,0x00,
+    0xff,0x03,
+    0xff,0x03
+};
+
+static const unsigned char resize_bits[] = {
+    0xff, 0x03, 
+    0xff, 0x03, 
+    0x33, 0x03, 
+    0x33, 0x03, 
+    0xf3, 0x03, 
+    0xf3, 0x03,
+    0x03, 0x03, 
+    0x03, 0x03, 
+    0xff, 0x03, 
+    0xff, 0x03
+};
+
+
 // If the maximize graphic above (which I did quickly in about a
 // minute, just so I could have something) doesn't please, maybe one
 // of the following would be better.  IMO it doesn't matter, as long
@@ -315,7 +407,7 @@ void NextClient::init()
     QHBoxLayout *windowLayout = new QHBoxLayout();
     mainLayout->addLayout(titleLayout);
     mainLayout->addLayout(windowLayout, 1);
-    mainLayout->addSpacing(isResizable() ? handleSize : 1);
+    mainLayout->addSpacing(mustDrawHandle() ? handleSize : 1);
 
     windowLayout->addSpacing(1);
     if (isPreview())
@@ -423,9 +515,19 @@ void NextClient::addButtons(QBoxLayout* titleLayout, const QString& spec)
                      this, SLOT(menuButtonPressed()) );
             break;
 
+        case 'L':
+            button[SHADE_IDX] =
+                new NextButton(this, "shade", NULL, 0, 0, i18n("Shade"));
+            titleLayout->addWidget( button[SHADE_IDX] );
+            connect( button[SHADE_IDX], SIGNAL(clicked()),
+                     this, SLOT(shadeClicked()) );
+            // NOTE DIFFERENCE: set the pixmap separately (2 states)
+	    shadeChange();
+            break;
+	    
         case 'S':
             button[STICKY_IDX] =
-                new NextButton(this, "sticky", NULL, 0, 0, i18n("On All Desktops"));
+                new NextButton(this, "sticky", NULL, 0, 0, i18n("On all desktops"));
             titleLayout->addWidget( button[STICKY_IDX] );
             connect( button[STICKY_IDX], SIGNAL(clicked()),
                      this, SLOT(toggleOnAllDesktops()) );
@@ -433,17 +535,48 @@ void NextClient::addButtons(QBoxLayout* titleLayout, const QString& spec)
 	    desktopChange();
             break;
 
-        case 'X':
-            if( isCloseable()) {
-            button[CLOSE_IDX] =
-                new NextButton(this, "close", close_bits, 10, 10,
-                               i18n("Close"));
-            titleLayout->addWidget( button[CLOSE_IDX] );
-            connect( button[CLOSE_IDX], SIGNAL(clicked()),
-                     this, SLOT(closeWindow()) );
+	case 'F':
+            button[ABOVE_IDX] = new NextButton(this, "above", NULL, 0, 0, "");
+            titleLayout->addWidget( button[ABOVE_IDX] );
+            connect( button[ABOVE_IDX], SIGNAL(clicked()),
+                     this, SLOT(aboveClicked()) );
+	    connect(this, SIGNAL(keepAboveChanged(bool)), 
+		    SLOT(keepAboveChange(bool)));
+	    keepAboveChange(keepAbove());
+	    break;
+	    
+	case 'B':
+            button[BELOW_IDX] = new NextButton(this, "below", NULL, 0, 0, "");
+            titleLayout->addWidget( button[BELOW_IDX] );
+            connect( button[BELOW_IDX], SIGNAL(clicked()),
+                     this, SLOT(belowClicked()) );
+	    connect(this, SIGNAL(keepBelowChanged(bool)), 
+		    SLOT(keepBelowChange(bool)));
+	    keepBelowChange(keepBelow());
+	    break;
+        
+	case 'X':
+            if (isCloseable()) {
+		button[CLOSE_IDX] =
+		    new NextButton(this, "close", close_bits, 10, 10,
+				   i18n("Close"));
+		titleLayout->addWidget(button[CLOSE_IDX]);
+		connect(button[CLOSE_IDX], SIGNAL(clicked()),
+			 this, SLOT(closeWindow()));
             }
             break;
 
+	case 'R':
+            if (mustDrawHandle()) {
+		button[RESIZE_IDX] =
+		    new NextButton(this, "resize", resize_bits, 10, 10,
+				   i18n("Resize"));
+		titleLayout->addWidget(button[RESIZE_IDX]);
+		// NOTE DIFFERENCE: capture pressed(), not clicked()
+		connect(button[RESIZE_IDX], SIGNAL(pressed()),
+			 this, SLOT(resizePressed()));
+            }
+	    break;
         case '_':
             // TODO: Add spacer handling
             break;
@@ -456,6 +589,16 @@ void NextClient::addButtons(QBoxLayout* titleLayout, const QString& spec)
     }
 }
 
+bool NextClient::mustDrawHandle() const 
+{ 
+    bool drawSmallBorders = !options()->moveResizeMaximizedWindows();
+    if (drawSmallBorders && (maximizeMode() & MaximizeVertical)) {
+	return false;
+    } else {
+	return isResizable();
+    }
+}
+
 void NextClient::iconChange()
 {
     if (button[MENU_IDX] && button[MENU_IDX]->isVisible())
@@ -466,10 +609,13 @@ void NextClient::menuButtonPressed()
 {
     // Probably don't need this null check, but we might as well.
     if (button[MENU_IDX]) {
-        QPoint menupoint ( button[MENU_IDX]->rect().bottomLeft().x()-1,
-                           button[MENU_IDX]->rect().bottomLeft().y()+2 );
+	QRect menuRect = button[MENU_IDX]->rect();
+        QPoint menuTop = button[MENU_IDX]->mapToGlobal(menuRect.topLeft());
+        QPoint menuBottom = button[MENU_IDX]->mapToGlobal(menuRect.bottomRight());
+	menuTop += QPoint(1, 1);
+	menuBottom += QPoint(1, 1);
         KDecorationFactory* f = factory();
-        showWindowMenu(button[MENU_IDX]->mapToGlobal(menupoint));
+        showWindowMenu(QRect(menuTop, menuBottom));
         if( !f->exists( this )) // 'this' was deleted
             return;
 	button[MENU_IDX]->setDown(false);
@@ -480,20 +626,32 @@ void NextClient::menuButtonPressed()
 void NextClient::maximizeButtonClicked()
 {
     if (button[MAXIMIZE_IDX]) {
-        switch (button[MAXIMIZE_IDX]->lastButton()) {
-        case MidButton:
-	    maximize(maximizeMode() ^ MaximizeVertical);
-            break;
-        case RightButton:
-	    maximize(maximizeMode() ^ MaximizeHorizontal);
-            break;
-        default:
-	    maximize(maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull);
-            break;
-        }
+        maximize(button[MAXIMIZE_IDX]->lastButton());
     }
 }
 
+void NextClient::shadeClicked()
+{
+    setShade(!isSetShade());
+}
+
+void NextClient::aboveClicked()
+{
+    setKeepAbove(!keepAbove());
+}
+
+void NextClient::belowClicked()
+{
+    setKeepBelow(!keepBelow());
+    keepAboveChange(keepAbove());
+    keepBelowChange(keepBelow());
+}
+
+void NextClient::resizePressed()
+{
+    performWindowOperation(ResizeOp);
+}
+
 void NextClient::resizeEvent(QResizeEvent *)
 {
     if (widget()->isVisible()) {
@@ -554,7 +712,7 @@ void NextClient::paintEvent( QPaintEvent* )
     p.drawText( t, AlignCenter | AlignVCenter, caption() );
 
     // Draw resize handle
-    if (isResizable()) {
+    if (mustDrawHandle()) {
         int corner = 16 + 3*handleSize/2;
 	qDrawShadePanel(&p,
 		fr.x() + 1, fr.bottom() - handleSize, corner-1, handleSize,
@@ -598,7 +756,7 @@ void NextClient::desktopChange()
     if (NextButton * b = button[STICKY_IDX]) {
         b->setBitmap( on ? unsticky_bits : sticky_bits, 10, 10);
 	QToolTip::remove(b);
-	QToolTip::add(b, on ? i18n("Not On All Desktops") : i18n("On All Desktops"));
+	QToolTip::add(b, on ? i18n("Not on all desktops") : i18n("On all desktops"));
     }
 }
 
@@ -611,7 +769,7 @@ void NextClient::maximizeChange()
 	QToolTip::add(button[MAXIMIZE_IDX],
 		m ? i18n("Restore") : i18n("Maximize"));
     }
-    //spacer->changeSize(10, isResizable() ? handleSize : 1,
+    //spacer->changeSize(10, mustDrawHandle() ? handleSize : 1,
     //		    QSizePolicy::Expanding, QSizePolicy::Minimum);
     //mainLayout->activate();
 }
@@ -656,11 +814,38 @@ void NextClient::borders(int &left, int &right, int &top, int &bottom) const
 {
     left = right = 1;
     top = titleHeight; // FRAME is this ok?
-    bottom = isResizable() ? handleSize : 1;
+    bottom = mustDrawHandle() ? handleSize : 1;
 }
 
 void NextClient::shadeChange()
 {
+    if (NextButton *b = button[SHADE_IDX]) {
+        b->setBitmap(isSetShade() ? unshade_bits : shade_bits, 10, 10);
+	QToolTip::remove(b);
+	QToolTip::add(b, isSetShade() ? i18n("Unshade") : i18n("Shade"));
+    }
+}
+
+void NextClient::keepAboveChange(bool above)
+{
+    if (NextButton *b = button[ABOVE_IDX]) {
+        b->setBitmap(above ? from_above_bits : keep_above_bits, 10, 10);
+	QToolTip::remove(b);
+	QToolTip::add(b, above ? 
+		i18n("Do not keep above others") : i18n("Keep above others"));
+	b->repaint(false);
+    }
+}
+
+void NextClient::keepBelowChange(bool below)
+{
+    if (NextButton *b = button[BELOW_IDX]) {
+        b->setBitmap(below ? from_below_bits : keep_below_bits, 10, 10);
+	QToolTip::remove(b);
+	QToolTip::add(b, below ? 
+		i18n("Do not keep below others") : i18n("Keep below others"));
+	b->repaint(false);
+    }
 }
 
 QSize NextClient::minimumSize() const
@@ -708,6 +893,22 @@ bool NextClient::eventFilter(QObject *o, QEvent *e)
     return false;
 }
 
+bool NextClient::drawbound(const QRect& geom, bool /* clear */)
+{
+    QPainter p(workspaceWidget());
+    p.setPen(QPen(Qt::white, 3));
+    p.setRasterOp(Qt::XorROP);
+    p.drawRect(geom);
+    int leftMargin = geom.left() + 2;
+    p.fillRect(leftMargin, geom.top() + titleHeight - 1,
+	    geom.width() - 4, 3, Qt::white);
+    if (mustDrawHandle()) {
+	p.fillRect(leftMargin, geom.bottom() - handleSize - 1,
+		geom.width() - 4, 3, Qt::white);
+    }	    
+    return true;
+}
+
 // =====================================
 
 NextClientFactory::NextClientFactory()
